<?php

/**
 * Admin settings page
 */
function inline_messages_admin_settings() {
    
 /**
  * General
  */
  $form['inline_messages_general'] = array(
    '#type'          => 'fieldset',
    '#title'         => t("General Settings"),
    '#collapsible'   => TRUE,
    '#collapsed'     => TRUE
  );
  
  // change the comment submit button text?
  $form['inline_messages_general']['inline_messages_form_comment_submit'] = array(
    '#type' => 'textfield',
    '#title' => 'Comment Form Submit Button Text',
    '#description' => t("Enter a custom label for the %what form submit button.", array('%what' => 'Comment')),
    '#default_value' => variable_get('inline_messages_form_comment_submit', ''),
  );
  
  if (!module_exists('comment')) {
    $form['inline_messages_general']['inline_messages_scrollto_path']['#disabled'] = TRUE;
  }
  
  // hide the preview button?
  $form['inline_messages_general']['inline_messages_hide_preview'] = array(
    '#type'           => 'checkbox',
    '#title'          => 'Hide preview button',
    '#default_value'  => variable_get('inline_messages_hide_preview', 0),
    '#description'    => t("Hide the preview button for all forms, including ones not using Inline Messages.")
  );

 /**
  * Forms
  */
  $form['inline_messages_forms'] = array(
    '#type'          => 'fieldset',
    '#title'         => t("Forms"),
    '#collapsible'   => TRUE,
    '#collapsed'     => TRUE
  );

  // add a use any module form
  $form['inline_messages_forms']['inline_messages_mforms_all'] = array(
    '#type' => 'checkbox',
    '#title' => 'All module forms',
    '#default_value' => variable_get('inline_messages_mforms_all', 0),
    '#description' => t("Display all form messages inline, including any created after these settings have been saved?<br/> Note: This does not include any forms on the administration pages.")
  );
  
  // put all the forms collected into checkboxes
  $form['inline_messages_forms']['inline_messages_mforms'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Module Forms',
    '#options' => inline_messages_get_forms(),
    '#default_value' => variable_get('inline_messages_mforms', array()),
    '#description' => t("If %all isn't selected, select which forms should display messages inline.", array('%all' => 'All module forms'))
  );
  
  /**
   * jQuery.ScrollTo Settings
   * http://flesler.blogspot.com/2007/10/jqueryscrollto.html
   */
  $form['inline_messages_scrollto'] = array(
    '#type'          => 'fieldset',
    '#title'         => t("jQuery.ScrollTo Settings"),
    '#description'   => t("You can add additional settings to add animation to page scrolling. Refer to the !link page for more information.", array('!link' => SCROLLTO_LINK)),
    '#collapsible'   => TRUE,
    '#collapsed'     => TRUE
  );
  
  // location of jQuery.ScrollTo plugin
  $form['inline_messages_scrollto']['inline_messages_scrollto_path'] = array(
    '#type' => 'textfield',
    '#title' => 'Path to jQuery.ScrollTo plugin',
    '#description' => t("This module is packaged with %file or you can specify a path relative to your site root.", array('%file' => 'jquery.scrollTo-1.4.2-min.js')),
    '#default_value' => variable_get('inline_messages_scrollto_path', SCROLLTO_PATH_DEFAULT),
  );    

  $form['inline_messages_scrollto']['inline_messages_scrollto_duration'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Duration"),
    '#default_value' => variable_get('inline_messages_scrollto_duration', ''),
    '#description'   => t("Length of the animation in milliseconds, none or 0 to jump immediately."),
  );
  $form['inline_messages_scrollto']['inline_messages_scrollto_offset'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Offset"),
    '#default_value' => variable_get('inline_messages_scrollto_offset', ''),
    '#description'   => t("Number of pixels (positive or negative) to be added to the final target position."),
  );
  $form['#validate'][] = 'inline_messages_settings_validate';
  return system_settings_form($form);
}

/**
 * Validate
 */
function inline_messages_settings_validate($form, &$form_state) {
  // make sure there's a valid path to the jQuery.ScrollTo plugin  
  if (!file_exists($form_state['values']['inline_messages_scrollto_path'])) {
    form_error($form['inline_messages_scrollto']['inline_messages_scrollto_path'], t('Enter a valid path to the !link plugin.', array('!link' => SCROLLTO_LINK)));
  }
  
  // validate the jQuery.ScrollTo Settings  
  if (!empty($form_state['values']['inline_messages_scrollto_duration']) && !is_numeric($form_state['values']['inline_messages_scrollto_duration'])) {
    form_error($form['inline_messages_scrollto']['inline_messages_scrollto_duration'], t("%setting must be numeric", array('%setting' => 'Duration')));
  }
  if (!empty($form_state['values']['inline_messages_scrollto_offset']) && !is_numeric($form_state['values']['inline_messages_scrollto_offset'])) {
    form_error($form['inline_messages_scrollto']['inline_messages_scrollto_offset'], t("%setting must be numeric", array('%setting' => 'Offset')));
  }
}

function inline_messages_get_forms() {
  // get all the modules that implement hook_form()
  $forms = array();
  foreach (module_implements('form') as $module) {
    // ignore some of the core modules
    $ignore = array('filter', 'taxonomy');
    if (!in_array($module, $ignore)) {
      $forms[$module .'_form'] = $module;
    }
  }
  
  // the core contact module doesn't implement hook_form(), so check for it
  if (module_exists('contact')) {
    $forms['contact_mail_page'] = 'contact page';
    $forms['contact_mail_user'] = 'contact user';
  }
  
  // sort the forms array alphabetically, preserving keys
  asort($forms);
  
  return $forms;
}